


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Model recognition          ;
;                            ;
; April 2011                 ;
;                            ;
; Katja Mehlhorn             ;
;                            ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; the model reads the name of a city, tries to retrieve it from memory and responds depending on whether or not it could retrieve it
; the parameters of this model were fit so that it fits the difference in retrieval times between recognized and unrecognized cities


;to run 1 trial (do-id n m) with n being the id of the trial and m being the participant number
;to run the whole experiment (do-many x) with x being the number of times you want to run it (x=1 runs all trials of each participant once)

(defun re () (run-environment))

;;;;;;;;;
;define variables and parameters

;to save the response
(defvar *response* nil)
(defvar *time* nil)
(defvar *rt* nil)
(defvar *run* 0)

;to run the code
(defparameter *trial-list* nil)
(defparameter *trials* nil)
(defparameter *curr-trial* nil)
(defparameter *trial-num* 0)
(defparameter *vp-nr* 0)
(defparameter *idm* 0)

;for the in- and output
(defparameter *results* nil)
(defparameter *read-write-path* (current-pathname))


;;;;;;;
(defun do-id (id vp)  ;IMPORTANT: only trials that are in the data can be called (e.g. trial 11)

  (reset)

  (setf *results* nil)
  (setf *vp-nr* 0)
  (setf *idm* 0)

  (read-file)                                ;reads in the file containing all trials

  (setf *trial-num* 0)

  (dolist (trial (nth (- vp 1) *trial-list*))       ;gets the trials of the first subjects from the trial-list
    (when (= (second (reverse trial)) id) 
      (setf *curr-trial* trial)))

  (load-vp (first (reverse *curr-trial*)))             ;calls load-vp: reads in the knowledge of the first vp
 
 (sdp :creation-time -1e10 :reference-count 3000000)  ;4.094

  (do-trial *curr-trial*)                       ;calls do-trial with the specified trial

)




(defun do-many (j)

  (setf *results* nil)
  (setf *vp-nr* 0)
  (setf *run* 0)

  (dotimes (k j)

  (read-file)

  (setf *idm* 0)
  (setf *run* (+ *run* 1))

  (dotimes (i (first (reverse (first (first (reverse *trial-list*))))))   ;number of VPn

              (setf *trials* (pop *trial-list*))  ;takes the trials of one vp and puts them in the list *trials*

              (do-vp (first (reverse (first *trials*))))))     ;calls do-vp for vp in the list
  
  (write-file-res)
)

;;;;;;;;;;;
;does all trials of all vpn

(defun do-exp ()

  (read-file)

  (setf *results* nil)
  (setf *vp-nr* 0)
  (setf *idm* 0)

  (dotimes (i (first (reverse (first (first (reverse *trial-list*))))))   ;number of VPn

              (setf *trials* (pop *trial-list*))  ;takes the trials of one vp and puts them in the list *trials*

              (do-vp (first (reverse (first *trials*)))))     ;calls do-vp for vp in the list

  (write-file-res)

)



;;;;;;
;does one vp - is called by do exp
(defun do-vp (vp)

 ; (dolist (i (nth (- vp 1) *trials*))   ;reads in all trials of the vp

  (dolist (i *trials*)

    (reset)                                 ;cannot reset in do-trial, as that would also reset dm!

    (setf *vp-nr* vp)
    (setf *idm* (+ *idm* 1))
    
    (load-vp vp)                            ;calls load-vp: reads in the knowledge of the vp
 
 (sdp :creation-time -1e10 :reference-count 3000000)  ;4.094


    (do-trial (setf *curr-trial* i)))       ;calls do-trial for each trial

)

;;;;;;;;;
;do trial - is called by do-id and by do-vp 

;present the two cities on the screen
;let the model look at the cities
;record the models decision

(defun do-trial (trial)

  (let (
         (text1 (first *curr-trial*))
         (window (open-exp-window "Cities" :visible nil))
         (start-time)
         )
    
    (add-text-to-exp-window :text text1 :x 25 :y 150)

    (setf *response* nil) 
    (setf *rt* nil)
    (setf *time* nil)

    (setf start-time (get-time))
    
    (if *actr-enabled-p*
        (progn
          (install-device window)
          (proc-display)
          (run 10 :real-time nil))
      
      (while (null *response*)
        (allow-event-manager window)))

    (setf *rt* (- *time* start-time))
    )

    
  (save-results-acc)


)


(defmethod rpm-window-key-event-handler ((win rpm-window) key)

  (setf *time* (get-time))

  (clear-exp-window)

  (when *actr-enabled-p* 
    (proc-display))

  (when (equalp (string key) "1")
    (setf *response* "R"))
  (when (equalp (string key) "2")
    (setf *response* "U"))


) 

;puts the current trial, including the modesl response in the list *results*
(defun save-results-acc ()

;  (let ((second (reverse *curr-trial*)))

  (push (list *run* *idm* *vp-nr* (second (reverse *curr-trial*)) (first *curr-trial*) (second *curr-trial*) (third *curr-trial*) *response* *rt*) *results*)

)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; input

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,
;trials
; read in trials as presented to participants 

(defun read-file ()
  (with-open-file (in (merge-pathnames 
                       (make-pathname :name "trials_recognition" :directory '(:relative "input") :type "txt")  
                       *read-write-path*)
                      :direction :input)
    (do ((trial-lis nil (cons trial trial-lis))
         (trial (read in nil 'end) (read in nil 'end)))
        ((equal trial 'end) (setf *trial-list* (reverse trial-lis))))
    )
 
  (restruc *trial-list*)

)


;puts all trials of one participant in a seperate list
(defun restruc (trial-lis)
 
 (let ((newlis) (trial) (act-vp 0))

  (dotimes (i (first (last (first (last *trial-list*)))) newlis) 
    (setf newlis (cons nil newlis)))

  (dotimes (j (length trial-lis) newlis)
    (setf trial (nth j trial-lis))
    (cond 
     ((= (first (last trial)) (+ act-vp 1)) (setf (nth act-vp newlis) (cons trial (nth act-vp newlis))))
     ((not (= (first (last trial)) (+ act-vp 1)))
      (progn 
        (setf (nth act-vp newlis) (reverse (nth act-vp newlis)))
        (incf act-vp)
        (setf (nth act-vp newlis) (cons trial (nth act-vp newlis))))) )
    )
  (setf (first (last newlis)) (reverse (first (last newlis))))
  (setf *trial-list* newlis))

)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,
;dm

;reads in the file containing the wm of the called vp and calls do trial
(defun load-vp (vp)

   (reset)

  (load-data (merge-pathnames 
              (make-pathname :name (format nil "complete_vp~a" vp) :directory '(:relative "input") :type "txt")  
              *read-write-path*))

)

;reads in the file containing the mw - is called by do-vp

(defun load-data (file)
  (with-open-file (in file)
    (progn (eval (read in)) t))
)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; output


(defun write-file-res ()

  (with-open-file (out (ensure-directories-exist 
                        (merge-pathnames 
                         (make-pathname :name "results_recognition" :directory '(:relative "output") :type "dat")  
                         *read-write-path*))
                       :direction :output :if-does-not-exist :create :if-exists :supersede)
    
    (setf *results* (reverse *results*))

    (push (list 'run 'id 'vp 'trial 'City 'Type 'rt_vp 'resp_model 'rt_model) *results*)  ;define the names of the variables in the output

    (dolist (line *results*)
      (format out "~{~a~t~}~%" line))
    )
)



(clear-all)

(define-model Cities

(sgp 
 :esc t                     ;enables subsymbolic computations
 :trace-detail low :v t     ;enables buffer trace
 :act nil                   ;enables activation trace
 :show-focus t              ;shows focus of attention

 :bll 0.5                   ;enables base-level learning and sets decay parameter to 0.5
 :ol 4                      ;4 = number of references for which bll is used, the rest uses optimized learning

 :rt -0.3                   ;set retrieval threshold
 :lf 0.1                    ;sets retrieval latency 

 :GA 0                      ;sets goal-activation to 0
 :IMAGINAL-ACTIVATION  1    ;sets goal-activation to 1      

 :MAS 3                     ;enables spreading activation - sets s = 3 

:er t                       ;if several chunks have same activation, choice is made randomly
:ans 0.2                    ;enables transient noice (is computed with each retrieval request)
:randomize-time 3           ;randomizes timing for motor and perception noice

:imaginal-delay .1          ;sets imaginal delay

:visual-attention-latency .035   ;sets attentipon latency

;:save-buffer-trace t       ;saves buffer trace in environment      


)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; declarative memory

;;; chunk types
(chunk-type city name value)
(chunk-type cue name value industry airport soccer)
(chunk-type complete name value industry airport soccer)
(chunk-type decide state)
(chunk-type decision city1 city2 industry airport soccer)

;;; chunks for control states

(add-dm
(start isa chunk)
(attend isa chunk)
(retrieve isa chunk)
(encode isa chunk)

(yes isa chunk)
(no isa chunk)
(unknown isa chunk)
)

;;; city chunks

(add-dm 
;(Aberdeen isa city name Aberdeen value "Aberdeen" Soccer yes Industry yes Airport yes)
;(Aanchester isa city name Aanchester value "Aanchester" Soccer no Industry no Airport no)
)

;;; start chunk

(add-dm (goal isa decide state start)) ; can be left out, if we use threaded cognition


;;; set baselevels

;(sdp :creation-time -1e12 :reference-count 1000000)   ; gives a baselevel of: 0.6931472
;(sdp :creation-time -1e12 :reference-count 2000000)   ; gives a baselevel of: 1.3862944

;;;preset hand locations - as participants have fingers on the keys

(set-hand-location left 1 2)
(set-hand-location right 2 2) 


;;; for the manual noise

;redefine the methods that determine the time for the execution


(defmethod compute-exec-time ((mtr-mod motor-module) (self punch))
 (setf (exec-time self) (max (init-time mtr-mod)
                             (randomize-time
                              (+ (init-time mtr-mod)
                                 (key-closure-time (current-device-interface)))))))


(defmethod compute-finish-time ((mtr-mod motor-module) (self punch))
 (max (exec-time self) (randomize-time (+ (init-time mtr-mod) (* 2 (burst-time mtr-mod))))))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; procedural memory



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;read city name

;find name of unattended city
(P find-unattended
   =goal>
      ISA         decide
      state       start
   ?visual-location>
      state       free
 ==>
   +visual-location>
      ISA         visual-location
      SCREEN-X    lowest         ;lets it read from left to right
      :attended    nil
   =goal>
      state       attend
)

;read city name
(P look-at-city
  =goal>
     ISA         decide
     state       attend
  =visual-location>
     ISA         visual-location
   ?visual>
     state       free
==>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve 
) 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;retrieve and encode meaning

;retrieve meaning of city
(p retrieve-city
 =goal>
    isa         decide
    state       retrieve
  =visual>
     isa         text
     value       =city
  ?retrieval>
     state       free
==>
  +retrieval>
     ISA         city
     value       =city
  =goal>
     state       encode 
)


;enter response
;if you can retrieve meaning, enter the known city,else the unknown

(P enter-known
  =goal>
     ISA         decide
     state       encode
  =retrieval>
     isa         city
     name       =city
  ?manual>
     state       free
==>
   +manual>
      ISA         punch
      hand        left 
      finger      index
   +visual>
    isa clear
  =goal>
     state       end
)

(P enter-unknown
  =goal>
     ISA         decide
     state       encode
  ?retrieval>
     state       error
  ?manual>
     state       free
==>
   +manual>
      ISA         punch
      hand        right 
      finger      index
   +visual>
    isa clear
  =goal>
     state       end
)



(goal-focus goal)
)

