

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Model 1&5.3.P.F           ;
;                            ;
; April 2011                 ;
;                            ;
; Katja Mehlhorn             ;
;                            ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;to run 1 trial (do-id n m) with n being the id of the trial and m being the participant number
;to run the whole experiment (do-many x) with x being the number of times you want to run it (x=1 runs all trials of each participant once)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;; definition of the environment with which ACT-R interacts;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun re () (run-environment))

;;;;;;;;;
;define variables and parameters

;to save the response
(defvar *response* nil)
(defvar *time* nil)
(defvar *rt* nil)
(defvar *run* 0)

;to run the code
(defparameter *trial-list* nil)
(defparameter *trials* nil)
(defparameter *curr-trial* nil)
(defparameter *trial-num* 0)
(defparameter *vp-nr* 0)
(defparameter *idm* 0)

;for the in- and output
(defparameter *results* nil)
(defparameter *read-write-path* (current-pathname))


;;;;;;;
(defun do-id (id vp)  ;IMPORTANT: only trials that are in the data can be called (e.g. trial 11)

  (reset)

  (setf *results* nil)
  (setf *vp-nr* 0)
  (setf *idm* 0)

  (read-file)                                ;reads in the file containing all trials

  (setf *trial-num* 0)

  (dolist (trial (nth (- vp 1) *trial-list*))       ;gets the trials of the first subjects from the trial-list
    (when (= (second (reverse trial)) id) 
      (setf *curr-trial* trial)))

  (load-vp (first (reverse *curr-trial*)))             ;calls load-vp: reads in the knowledge of the first vp

;set base-levels
  ;baselevels for city
  (sdp-fct (list (no-output (sdm isa city)) :creation-time -1e10 :reference-count 3000000))  ;4.094
 
;baselevels for cues, and sjis are read in with the dm-file


  (do-trial *curr-trial*)                       ;calls do-trial with the specified trial

)



(defun do-many (j)

  (setf *results* nil)
  (setf *vp-nr* 0)
  (setf *run* 0)

  (dotimes (k j)

  (read-file)

  (setf *idm* 0)
  (setf *run* (+ *run* 1))

  (dotimes (i (first (reverse (first (first (reverse *trial-list*))))))   ;number of VPn

              (setf *trials* (pop *trial-list*))  ;takes the trials of one vp and puts them in the list *trials*

              (do-vp (first (reverse (first *trials*))))))     ;calls do-vp for vp in the list
  
  (write-file-res)
)

;;;;;;;;;;;
;does all trials of all vpn

(defun do-exp ()

  (read-file)

  (setf *results* nil)
  (setf *vp-nr* 0)
  (setf *idm* 0)

  (dotimes (i (first (reverse (first (first (reverse *trial-list*))))))   ;number of VPn

              (setf *trials* (pop *trial-list*))  ;takes the trials of one vp and puts them in the list *trials*

              (do-vp (first (reverse (first *trials*)))))     ;calls do-vp for vp in the list

  (write-file-res)

)



;;;;;;
;does one vp - is called by do exp
(defun do-vp (vp)

  (dolist (i *trials*)

    (reset)                                 ;reset here, because cannot reset in do-trial, as that would also reset dm!

    (setf *vp-nr* vp)
    (setf *idm* (+ *idm* 1))
    
    (load-vp vp)                            ;calls load-vp: reads in the individual knowledge of the vp
 
;set base-levels
  ;baselevels for city
  (sdp-fct (list (no-output (sdm isa city)) :creation-time -1e10 :reference-count 3000000))  ;4.094
 
;baselevels for cues, and sjis are read in individually with the dm-file

    (do-trial (setf *curr-trial* i)))       ;calls do-trial for each trial

)

;;;;;;;;;
;do trial - is called by do-id and by do-vp 

;present the two cities on the screen
;let the model look at the cities
;record the models decision

(defun do-trial (trial)

  (let (
         (text1 (first *curr-trial*))
         (text2 (second *curr-trial*))
         (window (open-exp-window "Cities" :visible nil))
         (start-time)
         )
    
    (add-text-to-exp-window :text text1 :x 25 :y 150)
    (add-text-to-exp-window :text text2 :x 175 :y 150)
 

    (setf *response* nil) 
    (setf *rt* nil)
    (setf *time* nil)

    (setf start-time (get-time))
    
    (if *actr-enabled-p*
        (progn
          (install-device window)
          (proc-display)
          (run 30 :real-time nil))
      
      (while (null *response*)
        (allow-event-manager window)))

    (setf *rt* (- *time* start-time))
    )
    
  (save-results-acc)

)


(defmethod rpm-window-key-event-handler ((win rpm-window) key)

  (setf *time* (get-time))

  (clear-exp-window)

  (when *actr-enabled-p* 
    (proc-display))

  (when (equalp (string key) "1")
    (setf *response* "city1"))
  (when (equalp (string key) "2")
    (setf *response* "city2"))

) 

;puts the current trial, including the models response in the list *results*
(defun save-results-acc ()

  (push (list *run* *idm* *vp-nr* (second (reverse *curr-trial*)) (first *curr-trial*) (second *curr-trial*) (third *curr-trial*) (fourth *curr-trial*) (fifth *curr-trial*) *response* *rt*) *results*)

)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; input

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,
;trials
; read in trials

(defun read-file ()
  (with-open-file (in (merge-pathnames 
                       (make-pathname :name "trials_test" :directory '(:relative "input") :type "txt")  
                       *read-write-path*)
                      :direction :input)
    (do ((trial-lis nil (cons trial trial-lis))
         (trial (read in nil 'end) (read in nil 'end)))
        ((equal trial 'end) (setf *trial-list* (reverse trial-lis))))
    )
 
  (restruc *trial-list*)

)


;puts all trials of one participant in a seperate list
(defun restruc (trial-lis)
 
 (let ((newlis) (trial) (act-vp 0))

  (dotimes (i (first (last (first (last *trial-list*)))) newlis) 
    (setf newlis (cons nil newlis)))

  (dotimes (j (length trial-lis) newlis)
    (setf trial (nth j trial-lis))
    (cond 
     ((= (first (last trial)) (+ act-vp 1)) (setf (nth act-vp newlis) (cons trial (nth act-vp newlis))))
     ((not (= (first (last trial)) (+ act-vp 1)))
      (progn 
        (setf (nth act-vp newlis) (reverse (nth act-vp newlis)))
        (incf act-vp)
        (setf (nth act-vp newlis) (cons trial (nth act-vp newlis))))) )
    )
  (setf (first (last newlis)) (reverse (first (last newlis))))
  (setf *trial-list* newlis))

)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,
;dm

;reads in the file containing the wm of the called vp and calls do trial
;call vp 0 to read in the knowledge of the average vp

(defun load-vp (vp)

   (reset)

  (load-data (merge-pathnames 
              (make-pathname :name (format nil "complete_vp~a" vp) :directory '(:relative "input") :type "txt")  
              *read-write-path*))

)

;reads in the file containing the mw - is called by do-vp

(defun load-data (file)
  (with-open-file (in file)
    (progn (eval (read in)) t))
)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; output


(defun write-file-res ()

  (with-open-file (out (ensure-directories-exist 
                        (merge-pathnames 
                         (make-pathname :name "results_complete_1&5.3.P.F" :directory '(:relative "output") :type "dat")  
                         *read-write-path*))
                       :direction :output :if-does-not-exist :create :if-exists :supersede)
    
    (setf *results* (reverse *results*))

    (push (list 'run 'id 'vp 'trial 'name_city1 'name_city2 'pair2 'resp_vp 'rt_vp 'resp_model 'rt_model) *results*)  ;define the names of the variables in the output

    (dolist (line *results*)
      (format out "~{~a~t~}~%" line))
    )
)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;start of the model code


(clear-all)

(define-model Cities

(sgp 
 :esc t                     ;enables subsymbolic computations
 :trace-detail low :v nil     ;enables buffer trace
 :act nil                   ;enables activation trace
 :show-focus t              ;shows focus of attention

 :bll 0.5                   ;enables base-level learning and sets decay parameter to 0.5
 :ol 4                      ;4 = number of references for which bll is used, the rest uses optimized learning

 :rt -0.3                   ;set retrieval threshold
 :lf 0.1                    ;sets retrieval latency 

 :GA 0                      ;sets goal-activation to 0
 :IMAGINAL-ACTIVATION  1    ;sets goal-activation to 1      

 :MAS 3                     ;enables spreading activation - sets s = 3 

:er t                       ;if several chunks have same activation, choice is made randomly
:ans 0.2                    ;enables transient noice (is computed with each retrieval request)
:randomize-time 3           ;randomizes timing for motor and perception noice

:imaginal-delay .1          ;sets imaginal delay

:visual-attention-latency .035   ;sets attentipon latency

;:save-buffer-trace t       ;saves buffer trace in environment      

)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; declarative memory

;;; chunk types
(chunk-type city name value)
(chunk-type cue name value industry airport soccer)
(chunk-type complete name value industry airport soccer)
(chunk-type decide state)
(chunk-type decision city1 city2 industry airport soccer)

;;; chunks for control states

(add-dm

(start isa chunk)
(attend isa chunk)
(retrieve isa chunk)
(encode isa chunk)

(yes isa chunk)
(no isa chunk)
(unknown isa chunk)

(retrieve-cues isa chunk)
(retrieve-soccer isa chunk)
(retrieve-industry isa chunk)
(retrieve-airport isa chunk)

)

;;; add start chunk

(add-dm (goal isa decide state start)) ; can be left out, if we use threaded cognition

;all other chunks are read in individually for each pp

;;;preset hand locations - as participants have fingers on the keys

(set-hand-location left 1 2)
(set-hand-location right 2 2) 

;;; for the manual noise

;redefine the methods that determine the time for the execution

(defmethod compute-exec-time ((mtr-mod motor-module) (self punch))
 (setf (exec-time self) (max (init-time mtr-mod)
                             (randomize-time
                              (+ (init-time mtr-mod)
                                 (key-closure-time (current-device-interface)))))))


(defmethod compute-finish-time ((mtr-mod motor-module) (self punch))
 (max (exec-time self) (randomize-time (+ (init-time mtr-mod) (* 2 (burst-time mtr-mod))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; procedural memory



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;read city name

;find name of unattended city
(P find-unattended
   =goal>
      ISA         decide
      state       start
   ?visual-location>
      state       free
 ==>
   +visual-location>
      ISA         visual-location
      SCREEN-X    lowest         ;lets it read from left to right
      :attended    nil
   =goal>
      state       attend
)


;read city name
(P look-at-city
  =goal>
     ISA         decide
     state       attend
  =visual-location>
     ISA         visual-location
   ?visual>
     state       free
==>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve 
) 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;retrieve and encode meaning

;retrieve meaning of city
(p retrieve-city
 =goal>
    isa         decide
    state       retrieve
  =visual>
     isa         text
     value       =city
  ?retrieval>
     state       free
==>
  +retrieval>
     ISA         city
     value       =city
  =goal>
     state       encode 
)

;encode first city known
(P encode-first-city-known
  =goal>
     ISA         decide
     state       encode
  =retrieval>
     isa         city
     name       =city
  ?imaginal>
     state       free
     buffer      empty
==>
  +imaginal>
     ISA         decision
     city1       =city     
  =goal>
     state      start
  +visual>
     isa clear
)

;encode first city unknown
(P encode-first-city-unknown
  =goal>
     ISA         decide
     state       encode
  ?retrieval>
     state       error
  ?imaginal>
     state       free
     buffer      empty
==>
  +imaginal>
     isa         decision
     city1       unknown     
  =goal>
     state       start
  +visual>
   isa clear
)

;encode second city known
(P encode-second-city-known
  =goal>
     ISA         decide
     state       encode
  =retrieval>
     isa         city
     name       =city
  =imaginal>
     isa         decision
     city1       =firstcity
     city2       nil
==>
  +imaginal>
     isa         decision
      city2      =city     ;encode one in one and two in two - neccessary for response!
      city1      =firstcity
  =goal>
     state       retrieve-cues
  +visual>
   isa clear
)

;encode second city unknown
(P encode-second-city-unknown
  =goal>
     ISA         decide
     state       encode
  ?retrieval>
     state       error
  =imaginal>
     isa         decision
     city1       =city
     city2       nil
==>
  +imaginal>
     isa         decision
     city1       =city
     city2       unknown
  =goal>
     state       retrieve-cues
  +visual>
   isa clear
)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;retrieve cues

;soccer

(p retrieve-left-soccer-yes
   =goal>
      isa        decide
     state       retrieve-cues
   ?imaginal>
      state      free
   =imaginal>
      isa        decision
      city1      =city
      - city1    unknown   ;so it only fires if there is a known city in that slot
      - city2     nil      ;and both cities have been tried to retrieve
     soccer        nil
   ?retrieval>
      state      free
==>
  =imaginal> 
  +retrieval>
      isa        cue
      name       =city
      soccer      yes
   =goal>
      state     retrieve-soccer
)

(p retrieve-right-soccer-yes
   =goal>
      isa        decide
     state       retrieve-cues
   ?imaginal>
      state      free
   =imaginal>
      isa        decision
      city2      =city
      - city2    unknown   ;so it only fires if there is a known city in that slot
      - city1     nil      ;and both cities have been tried to retrieve
     soccer        nil
   ?retrieval>
      state      free
==>
  =imaginal> 
  +retrieval>
      isa        cue
      name       =city
      soccer      yes
   =goal>
      state     retrieve-soccer
)

(p encode-soccer-yes
  =goal>
     ISA         decide
      state     retrieve-soccer
  =retrieval>
     isa         cue
     soccer      yes
  =imaginal>
     isa         decision
     city1       =city1     ;if both cities have been checked and cue one hasn't been checked yet
     city2       =city2
     soccer        nil
  ?imaginal>
     state       free
==>
  =retrieval>
  +imaginal>           ;in this model cues are encoded with time cost
      city1      =city1
      city2      =city2
      soccer       yes
   =goal>
     state       retrieve-cues
)

(p encode-soccer-unknown
  =goal>
     ISA         decide
      state     retrieve-soccer
  ?retrieval>
      state     error
  =imaginal>
     isa         decision
     city1       =city1     ;if both cities have been checked and cue one hasn't been checked yet
     city2       =city2
     soccer        nil
  ?imaginal>
     state       free
==>
  =retrieval>
  +imaginal>
      city1      =city1
      city2      =city2
      soccer     unknown
   =goal>
     state       retrieve-cues
)


;industry

(p retrieve-left-industry-yes
   =goal>
      isa        decide
     state       retrieve-cues
   ?imaginal>
      state      free
   =imaginal>
      isa        decision
      city1      =city
      - city1    unknown   ;so it only fires if there is a known city in that slot
      - city2     nil      ;and both cities have been tried to retrieve
     industry        nil
   ?retrieval>
      state      free
==>
  =imaginal> 
  +retrieval>
      isa        cue
      name       =city
      industry        yes
   =goal>
      state     retrieve-industry
)

(p retrieve-right-industry-yes
   =goal>
      isa        decide
     state       retrieve-cues
   ?imaginal>
      state      free
   =imaginal>
      isa        decision
      city2      =city
      - city2    unknown   ;so it only fires if there is a known city in that slot
      - city1     nil      ;and both cities have been tried to retrieve
     industry        nil
   ?retrieval>
      state      free
==>
  =imaginal> 
  +retrieval>
      isa        cue
      name       =city
      industry        yes
   =goal>
      state     retrieve-industry
)


(p encode-industry-yes
  =goal>
     ISA         decide
      state     retrieve-industry
  =retrieval>
     isa         cue
     industry      yes
  =imaginal>
     isa         decision
     city1       =city1     ;if both cities have been checked and cue one hasn't been checked yet
     city2       =city2
     industry        nil
  ?imaginal>
     state       free
==>
  =retrieval>
  +imaginal>
      city1      =city1
      city2      =city2
      industry       yes
   =goal>
     state       retrieve-cues
)

(p encode-industry-unknown
  =goal>
     ISA         decide
      state      retrieve-industry
  ?retrieval>
      state     error
  =imaginal>
     isa         decision
     city1       =city1     ;if both cities have been checked and cue one hasn't been checked yet
     city2       =city2
     industry        nil
  ?imaginal>
     state       free
==>
  =retrieval>
  +imaginal>
      city1      =city1
      city2      =city2
      industry   unknown
   =goal>
     state       retrieve-cues
)


;airport

(p retrieve-left-airport-yes
   =goal>
      isa        decide
     state       retrieve-cues
   ?imaginal>
      state      free
   =imaginal>
      isa        decision
      city1      =city
      - city1    unknown   ;so it only fires if there is a known city in that slot
      - city2     nil      ;and both cities have been tried to retrieve
     airport        nil
   ?retrieval>
      state      free
==>
  =imaginal> 
  +retrieval>
      isa        cue
      name       =city
      airport        yes
   =goal>
      state     retrieve-airport
)

(p retrieve-right-airport-yes
   =goal>
      isa        decide
     state       retrieve-cues
   ?imaginal>
      state      free
   =imaginal>
      isa        decision
      city2      =city
      - city2    unknown   ;so it only fires if there is a known city in that slot
      - city1     nil      ;and both cities have been tried to retrieve
     airport        nil
   ?retrieval>
      state      free
==>
  =imaginal> 
  +retrieval>
      isa        cue
      name       =city
      airport     yes
   =goal>
      state     retrieve-airport
)

(p encode-airport-yes
  =goal>
     ISA         decide
      state     retrieve-airport
  =retrieval>
     isa         cue
     airport      yes
  =imaginal>
     isa         decision
     city1       =city1     ;if both cities have been checked and cue one hasn't been checked yet
     city2       =city2
     airport        nil
  ?imaginal>
     state       free
==>
  =retrieval>
  +imaginal>
      city1      =city1
      city2      =city2
      airport       yes
   =goal>
     state       retrieve-cues
)

(p encode-airport-unknown
  =goal>
     ISA         decide
      state     retrieve-airport
  ?retrieval>
      state     error
  =imaginal>
     isa         decision
     city1       =city1     ;if both cities have been checked and cue one hasn't been checked yet
     city2       =city2
     airport        nil
  ?imaginal>
     state       free
==>
  =retrieval>
  +imaginal>
      city1      =city1
      city2      =city2
      airport    unknown
   =goal>
     state       retrieve-cues
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;respond - recognition

;they don't check if cues were retrieved, so they can fire directly after the cities were encoded, or after one, two, three cues were encoded

;if you know the first, but not the second city, respond with left 

(p respond-left
  =goal>
     ISA         decide
      state      retrieve-cues
  =imaginal>
     isa         decision
     city1       =city
     city2       unknown
;     - soccer       nil
;     - industry     nil
;     - airport      nil
   ?manual>   
      state           free
   ?imaginal>
      state           free
 ==>
   +manual>
      ISA         punch
      hand        left 
      finger      index
   +visual>
    isa clear
)

;if you know the second, but not the first city, respond with right
(p respond-right
  =goal>
     ISA         decide
      state      retrieve-cues
  =imaginal>
     isa         decision
     city2       =city
     city1       unknown
 ;    - soccer       nil
 ;    - industry     nil
 ;    - airport      nil
   ?imaginal>
      state           free
   ?manual>   
      state           free
==>
   +manual>
      ISA         punch
      hand        right 
      finger      index
   +visual>
    isa clear
)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;respond

;;; as soon as you encoded THREE positive cues, enter the known city

(p respond-known3-left
  =goal>
     ISA         decide
     state       retrieve-cues
  =imaginal>
     isa         decision
     city1       =city
     city2       unknown
     soccer        yes
     industry      yes
     airport       yes
 ?manual>   
      state       free
   ?imaginal>
      state           free
==>
   +manual>
      ISA         punch
      hand        left 
      finger      index   
   +visual>
    isa clear
)


(p respond-known3-right
  =goal>
     ISA         decide
     state       retrieve-cues
  =imaginal>
     isa         decision
     city2       =city
     city1       unknown
     soccer        yes
     industry      yes
     airport       yes
 ?manual>   
      state       free
   ?imaginal>
      state           free
==>
   +manual>
      ISA         punch
      hand        right 
      finger      index   
   +visual>
    isa clear
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;the special case to read and encode the cues again after you encoded the cues
;this production can only fire if at least two cues were encoded 

;it starts with retrieving the cues again

(P read-again1
   =goal>
      ISA         decide
      state       retrieve-cues
  =imaginal>
     isa         decision
     - city2       nil
     - city1       nil
     - soccer       nil
     - industry       nil
     airport       nil
   ?imaginal>
      state           free
 ==>
  +imaginal>
     soccer       nil
     industry     nil
     airport      nil
   +visual>
    isa clear
)

(P read-again2
   =goal>
      ISA         decide
      state       retrieve-cues
  =imaginal>
     isa         decision
     - city2       nil
     - city1       nil
     - soccer       nil
     - airport       nil
     industry       nil
   ?imaginal>
      state           free
 ==>
  +imaginal>
     soccer       nil
     industry     nil
     airport      nil
   +visual>
    isa clear
)

(P read-again3
   =goal>
      ISA         decide
      state       retrieve-cues
  =imaginal>
     isa         decision
     - city2       nil
     - city1       nil
     - industry       nil
     - airport       nil  
     soccer       nil  
   ?imaginal>
      state           free
 ==>
  +imaginal>
     soccer       nil
     industry     nil
     airport      nil
   +visual>
    isa clear
)

(P read-again
   =goal>
      ISA         decide
      state       retrieve-cues
  =imaginal>
     isa         decision
     - city2       nil
     - city1       nil
     - soccer      nil
     - industry    nil
     - airport     nil
   ?imaginal>
      state           free
 ==>
  +imaginal>
     soccer       nil
     industry     nil
     airport      nil
   +visual>
    isa clear
)



(goal-focus goal)
)

